package com.persagy.apm.alarmservice.project.alarmcondition.controller;

import com.persagy.apm.alarmservice.project.alarmcondition.model.Formula;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;

import com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition;
import com.persagy.apm.alarmservice.project.alarmcondition.model.ConvertAlarmConditionTool;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.*;
import com.persagy.apm.alarmservice.project.alarmcondition.model.vo.AlarmConditionItemVO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.vo.AlarmConditionListItemVO;
import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.*;

/**
 * 报警条件控制层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:30:38
 */
@Api(tags = "报警条件")
@Validated
@RestController
@RequestMapping("alarmConditions")
public class AlarmConditionController {
    @Autowired
    private IAlarmConditionService alarmConditionService;

    @ApiOperation(value = "创建报警条件")
    @ApiIgnore
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createAlarmCondition(
            @Valid @RequestBody AddAlarmConditionDTO addAlarmConditionDTO) {
        Map<String, Object> alarmCondition = alarmConditionService.createAlarmCondition(addAlarmConditionDTO);
        return ResultHelper.single(new ApmResponseUpsertVO((String) alarmCondition.get("id")));
    }

    @ApiOperation(value = "报警条件详情")
    @ApiIgnore
    @PostMapping("/get")
    public CommonResult<AlarmConditionItemVO> queryAlarmConditionDetail(
            @Valid @RequestBody QueryAlarmConditionDetailDTO queryAlarmConditionDetailDTO) {
        AlarmCondition alarmCondition = alarmConditionService.queryAlarmConditionDetail(queryAlarmConditionDetailDTO.getId());
        AlarmConditionItemVO alarmConditionItemVO = ConvertAlarmConditionTool.INSTANCE.convert2ResponseItemDTO(alarmCondition);
        return ResultHelper.single(alarmConditionItemVO);
    }


    @ApiOperation(value = "更新报警条件")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateAlarmCondition(
            @Valid @RequestBody UpdateAlarmConditionDTO updateAlarmConditionDTO) {
        alarmConditionService.updateAlarmCondition(updateAlarmConditionDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateAlarmConditionDTO.getId()));
    }


    @ApiOperation(value = "删除报警条件")
    @ApiIgnore
    @PostMapping("/delete")
    public CommonResult deleteAlarmCondition(
            @RequestBody @Valid DeleteAlarmConditionDTO deleteAlarmConditionDTO) {
        alarmConditionService.deleteAlarmCondition(deleteAlarmConditionDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "报警条件列表")
    @ApiIgnore
    @PostMapping("/query")
    public CommonResult<PageList<AlarmConditionListItemVO>> queryAlarmCondition(
            @Valid @RequestBody QueryAlarmConditionDTO queryAlarmConditionDTO) {
        List<AlarmCondition> alarmConditionList = alarmConditionService.queryAlarmConditionList(queryAlarmConditionDTO);
        List<AlarmConditionListItemVO> resultList = ConvertAlarmConditionTool.INSTANCE.convert2List(alarmConditionList);
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "报警条件合法性校验")
    @PostMapping("/validate")
    public CommonResult<Boolean> validate(
            @Valid @RequestBody ValidateAlarmConditionDTO validateAlarmConditionDTO) {
        List<Formula> formula = validateAlarmConditionDTO.getFrontend().getFormula();
        return alarmConditionService.validate(validateAlarmConditionDTO);

    }
    //@ApiOperation(value = "报警条件列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<AlarmConditionListItemVO>> queryAlarmCondition(
    //        @Valid @RequestBody PageQueryAlarmConditionDTO pageQueryAlarmConditionDTO) {
    //    IPage<AlarmCondition> pagedList = alarmConditionService.pageQueryAlarmCondition(pageQueryAlarmConditionDTO);
    //    IPage<AlarmConditionListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertAlarmConditionTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，AlarmCondition实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}


}

